<?php
/**
 * Module "importorder" for Prestashop 1.6.0.9
 *
 * @author Ondřej Doněk, <ondrejd@gmail.com>
 * @link http://doc.prestashop.com/display/PS16/Creating+a+first+module
 */

// ---- Na stránce s formulářem skrýt `column_block`.
// TODO Povolit možnost AJAXu na stránce s formulářem (`PREF_AJAX`).
// TODO Povolit možnost rychlé objednávky pouze pro přihlášené uživatele (`PREF_REGUSERS`).
//      Pozn. zjišťování zda je uživatel: `if (($this->context->customer instanceof Customer)) { ... }`
// TODO Ukládat seznam nahraných souborů (případně i s ID uživatele a časem).
// TODO V administraci zobrazovat seznam nahraných souborů (s id uživatele a časem).
// TODO V nastavení nebudou texty ze šablon - ty budou přístupné přes lokalizaci.
// TODO V administraci musi byt moznost nahrat rozsireny soubor (vice sloupečků).
// TODO Ve front controlleru se mi texty nepropisují do možnosti lokalizace.

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Main module class.
 *
 * @author Ondřej Doněk, <ondrejd@gmail.com>
 */
class ImportOrder extends Module/*ModuleCore*/
{
    const PREF_AJAX = 'PS_IO_AJAX';
    const PREF_REGUSERS = 'PS_IO_REGUSERS';
    const PREF_IDENT = 'PS_IO_IDENT';

    const IDENT_TYPE_REF = 'reference';
    const IDENT_TYPE_EAN = 'ean13';
    const IDENT_TYPE_UPC = 'upc';

    /**
     * @return void
     */
    public function __construct()
    {
        $this->name = 'importorder';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'Ondřej Doněk';
        $this->need_instance = 0;
        //$this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->_directory = dirname(__FILE__);

        parent::__construct();

        $this->displayName = $this->l('Import Order');
        $this->description = $this->l('Module for enabling quick order using importing uploaded CSV or XLS files.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    /**
     * @return boolean
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        Configuration::updateValue(self::PREF_AJAX, 0);
        Configuration::updateValue(self::PREF_REGUSERS, 0);
        Configuration::updateValue(self::PREF_IDENT, self::IDENT_TYPE_REF);

        $this->registerHook('header');
        //$this->registerHook('footer');
        //$this->registerHook('leftColumn');
        //$this->registerHook('rightColumn');
        $this->registerHook('top');

        return true;
    }

    /**
     * @return boolean
     */
    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }

        Configuration::deleteByName(self::PREF_AJAX);
        Configuration::deleteByName(self::PREF_REGUSERS);
        Configuration::deleteByName(self::PREF_IDENT);

        return true;
    }

    /**
     * @return void Process module's configuration form.
     * @uses ImportOrder::getContentForm
     */
    public function getContent()
    {
        if (Tools::isSubmit('submitImportOrder')) {
            $pref_ajax = (int)Tools::getValue(self::PREF_AJAX);
            Configuration::updateValue(self::PREF_AJAX, $pref_ajax);
            
            $pref_regusers = (int)Tools::getValue(self::PREF_REGUSERS);
            Configuration::updateValue(self::PREF_REGUSERS, $pref_regusers);
            
            $pref_regusers = (string)Tools::getValue(self::PREF_IDENT);
            if (!in_array($pref_regusers, array(self::IDENT_TYPE_REF, self::IDENT_TYPE_EAN, self::IDENT_TYPE_UPC))) {
                $this->html .= $this->displayError($this->l('Invalid value for identificator type.'));
            } else {
                Configuration::updateValue(self::PREF_IDENT, $pref_regusers);
            }

            $this->html .= $this->displayConfirmation($this->l('Settings updated'));

            Tools::clearCache(Context::getContext()->smarty, $this->getTemplatePath('column_block.tpl'));
            Tools::redirectAdmin(
                $this->context->link->getAdminLink('AdminModules', true).
                '&conf=6'.$this->getContentUrlSuffix()
            );
        }

        return $this->getContentForm();
    }

    /**
     * @return void Renders module's configuration form
     * @uses ImportOrder::getContentFormValues
     * @uses ImportOrder::getFormSwitchValues
     */
    public function getContentForm()
    {
        $helper = new HelperForm();
        $helper->submit_action = 'submitImportOrder';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).$this->getContentUrlSuffix();
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array('fields_value' => $this->getContentFormValues());

        $fields = array();
        $fields[] = array(
            'type' => 'select',
            'label' => $this->l('Product identificator type'),
            'name' => self::PREF_IDENT,
            'desc' => $this->l('Select by which value should be product defined in imported files.'),
            'options' => array(
                'query' => array(
                    array(
                        'value' => self::IDENT_TYPE_REF,
                        'label' => $this->l('Code')
                    ),
                    array(
                        'value' => self::IDENT_TYPE_EAN,
                        'label' => $this->l('EAN')
                    ),
                    array(
                        'value' => self::IDENT_TYPE_UPC,
                        'label' => $this->l('UPC')
                    )
                ),
                'id' => 'value',
                'name' => 'label'
            )
            //'values' => $this->getFormSwitchValues('ps_io_regusers')
        );
        $fields[] = array(
            'type' => 'switch',
            'label' => $this->l('Use AJAX'),
            'name' => self::PREF_AJAX,
            'desc' => $this->l('Activate AJAX mode for Import Order module (compatible with the default theme)'),
            'values' => $this->getFormSwitchValues(strtolower(self::PREF_AJAX))
        );
        $fields[] = array(
            'type' => 'switch',
            'label' => $this->l('Only for registered users'),
            'name' => self::PREF_REGUSERS,
            'desc' => $this->l('Activate Import Order only for registered and logged-in users.'),
            'values' => $this->getFormSwitchValues(strtolower(self::PREF_REGUSERS))
        );

        return $this->html.$helper->generateForm(array(
            array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Settings'),
                        'icon' => 'importorder'
                    ),
                    'input' => $fields,
                    'submit' => array(
                        'title' => $this->l('Save')
                    )
                )
            )
        ));
    }

    /**
     * @return array Returns values from submitted configuration form.
     */
    public function getContentFormValues()
    {
        $values = array();
        $values[self::PREF_AJAX] = (int)Tools::getValue(self::PREF_AJAX, (int)Configuration::get(self::PREF_AJAX));
        $values[self::PREF_REGUSERS] = (int)Tools::getValue(self::PREF_REGUSERS, (int)Configuration::get(self::PREF_REGUSERS));
        $values[self::PREF_IDENT] = Tools::getValue(self::PREF_IDENT, Configuration::get(self::PREF_IDENT));

        return $values;
    }

    /**
     * @param string $id
     * @return array
     */
    private function getFormSwitchValues($id) {
        return array(
            array(
                'id' => $id.'_on',
                'value' => 1,
                'label' => $this->l('Yes')
            ),
            array(
                'id' => $id.'_off',
                'value' => 0,
                'label' => $this->l('No')
            )
        );
    }

    /**
     * @return string Returns suffix for admin URL with module's configuration page.
     */
    public function getContentUrlSuffix()
    {
        return '&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
    }

    /**
     * @param array $params
     * @return string
     */
    public function hookDisplayHeader($params)
    {
        if (!isset($this->context->controller->php_self)) {
            return;
        }

        $this->context->controller->addCss($this->_path . 'css/importorder.css');
        $this->context->controller->addJS($this->_path . 'js/importorder.js');

        return $this->display(
            __FILE__,
            'header.tpl',
            $this->getCacheId('importorder|header')
        );
    }

    /**
     * @param array $params
     * @return string
     */
    /*public function hookDisplayFooter($params)
    {
        return $this->displayColumnBlock($params);
    }*/

    /**
     * @param array $params
     * @return string
     */
    /*public function hookDisplayLeftColumn($params)
    {
        return $this->displayColumnBlock($params);
    }*/

    /**
     * @param array $params
     * @return string
     */
    /*public function hookDisplayRightColumn($params)
    {
        return $this->displayColumnBlock($params);
    }*/

    /**
     * @param array $params
     * @return string
     */
    public function hookDisplayTop($params)
    {
		// Skip if "importorder" form is displayed
		if (($this->context->controller instanceof importorderformModuleFrontController)) {
			return '';
		}

        return $this->displayColumnBlock($params);
    }

    /**
     * @param array $params
     * @return string
     */
    protected function displayColumnBlock($params)
    {
        if (!$this->isCached('column_block.tpl', $this->getCacheId('importorder|column_block'))) {
            $this->context->smarty->assign(array(
                'link_url' => $this->context->link->getModuleLink('importorder', 'form')
            ));
        }

        return $this->display(
            __FILE__,
            'column_block.tpl',
            $this->getCacheId('importorder|column_block')
        );
    }
}
